/**********************************************************************\
*                                                                      *
*      LOADER.C                                                        *
*                                                                      *
*  This firmware is for the Membership Card Basic Hex Loader           *
*                                                                      *
*        Developed By: MARKE Systems Inc.  
* Mark Moulding as "urrossum", posted in cosmacelf Mar 14 2012         *
*                                                                      *
\**********************************************************************/

#define VERSION 10                    /* 13-Mar-2012 */
/* Initial release */

#define DEBUG TRUE

#pragma SMALL
#include <REG51.H>

/* Useful constants and typedefs */
#define EOF -1
#define FALSE 0
#define TRUE !FALSE
#define FOREVER while(1)
#define HIGH 1
#define LOW 0

#define CAN 0x18
#define LF 0x0A
#define CR 0x0D
#define SP 0x20

#define TIMER_HIGH   0x59  /* 4 KHz ticks @ 8MHz */

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;

/* Tunable constants */
#define DELAY_FACTOR   44   /* 1 mSec delay time at 8 mHz */
#define TXBUFSIZE      40   /* size of data output buffer */
#define SDELAY         10   /* delay for SDelay() ~250 uSec */

/* Absolute variable locations (other than defined in REG51.H) */
sfr DataPort     = 0x90;       /* 8-bit port to card data lines */

sbit HostRxD     = P3^0;      /* UART data receive from host */
sbit Spare31     = P3^1;      /* (TxD) */
sbit Spare32     = P3^2;      /* (Int0) */
sbit Spare33     = P3^3;      /* (Int1) */
sbit RedLed      = P3^4;      /* LOW to light Red LED (T0) */
sbit GreenLed    = P3^5;      /* LOW to light Green LED (T1) */
sbit Spare36     = P3^6;      /* (Analog comparator output) */
sbit Strobe      = P3^7;      /* LOW for data load strobe */

/* Function prototypes */
void Timer0(void);                /* handle timer 0 interrupt */
void Serial(void);                /* handle serial port interrupt */
void ResetCommand(void);
void Delay(dword Wait);           /* Pauses for 'Wait' mSec */
void SDelay(void);                /* Pauses for SDELAY loops */

/* Global variables */
volatile byte Ticked;             /* Incremented by time-of-day timer */

/**********\
|*        *|
|* main() *|
|*        *|
\**********/

void main(void)
{
/* Variables for main loop */
  byte Value;             /* for general calculations and parameter passing */
  byte Iter;              /* for loop counter */
  byte InChr;             /* received serial character */
  byte CheckSum;          /* running calculated checksum */
  bit FirstHex;           /* TRUE when first char of hex pair received */
  byte TxBuf[TXBUFSIZE];  /* data output buffer */
  byte BufLen;            /* number of bytes to be sent */
  byte RecLen;            /* length of record to be received - not used */
  byte ReadPos;           /* position of (2-char hex) byte in input line */
  byte NewByte;           /* new byte being assembled */
  bit GotByte;            /* TRUE when a full byte has been assembled */
  word Address;           /* load address - not used */
  byte RecType;           /* type of record received - only data and end used */

/* Hardware initialization */
  EA = 0;                 /* make sure all interrupts are off */

  TCON=0x0; TMOD=0x22;         /* Set up timers:
                                        Tim0 = 8-bit auto-reload timer
                                               (used for clock ticks )
                                        Tim1 = 8-bit auto-reset timer
                                               (used for baud rate) */
/* Set up timer 0 for clock tick interrupt */
  TH0 = TIMER_HIGH;
  TR0 = 1;            /* start timer 0 */

/* Set up timer 1 for baud rate interrupt */
  TH1 = 0xEF;     /* ~1200 baud at 8MHz */
  TL1 = 0;
  TR1=1;          /* start timer 1 */

  PCON = 0x00;                  /* Power ctl = normal, x1 baud */
  SCON = 0x50;                  /* Serial: 8-bit UART, rec enabled */

/* Configure interrupts */
  ES=1; ET1=0; EX1=0; ET0=1; EX0=0;  /* Enable interrupts
                                        Ext0 = off
                                        Ext1 = off
                                        Tim0 = ON
                                        Tim1 = off
                                        Ser  = ON      */

  PS=1; PT1=0; PX1=0; PT0=0; PX0=0;  /* Interrupt priority:
                                        Serial = HIGH
                                        Timer1 = low
                                        Extrn1 = low
                                        Timer0 = low
                                        Extrn0 = low */

/* Initialize buffer pointers, etc. */
  Ticked = 0;
  FirstHex = FALSE;
  BufLen = 0;
  CheckSum = 0;
  GotByte = FALSE;

  EA = 1;

/* Announce presence to world and initialize YMZ chip */
  GreenLed = 0;
  Delay(250);
  RedLed = 0;
  Delay(250);
  GreenLed = 1;
  Delay(250);
  RedLed = 1;

  FOREVER
  {

/* Handle serial port received characters */
  if (RI)
  {
    InChr = SBUF & 0x7F;
    RI = FALSE;
    {
      if (InChr == ':')
      {
        FirstHex = FALSE;
        BufLen = 0;
        CheckSum = 0;
        ReadPos = 0;
        GotByte = FALSE;
        RedLed = TRUE;
        GreenLed = TRUE;
      }
      else if (InChr == CR)
      {
        if (CheckSum != 0)
            RedLed = FALSE;
        if (RecType == 0)
        {
          GreenLed = FALSE;
          if (BufLen > 0)
              BufLen -= 1;
          for (Iter=0; Iter<BufLen; Iter += 1)
          {
            DataPort = TxBuf[Iter];
            Strobe = FALSE;
            SDelay();
            Strobe = TRUE;
            SDelay();
          }
          DataPort = 0xFF;  /* release the port after writing */
        }
      }
      else if ((InChr >= '0') && (InChr <= '9'))
      {
        if (FirstHex)
        {
          NewByte = (NewByte << 4) | (InChr - '0');
          FirstHex = FALSE;
          GotByte = TRUE;
        }
        else
        {
          NewByte = InChr - '0';
          FirstHex = TRUE;
        }
      }
      else if ((InChr >= 'A') && (InChr <= 'F'))
      {
        if (FirstHex)
        {
          NewByte = (NewByte << 4) | (InChr - 'A' + 10);
          FirstHex = FALSE;
          GotByte = TRUE;
        }
        else
        {
          NewByte = InChr - 'A' + 10;
          FirstHex = TRUE;
        }
      }
      if (GotByte)
      {
        ReadPos += 1;
        CheckSum += NewByte;
        if (ReadPos == 1)       /* got record length */
        {
          RecLen = NewByte;
        }
        else if (ReadPos == 2)  /* got load address high byte */
        {
          Address = NewByte;
        }
        else if (ReadPos == 3)  /* got load address low byte */
        {
          Address = (Address << 8) + NewByte;
        }
        else if (ReadPos == 4)  /* got record type */
        {
          RecType = NewByte;
        }
        else  /* must be a data byte, or checksum */
        {
          TxBuf[BufLen] = NewByte;
          BufLen += 1;
        }
        GotByte = FALSE;
      }
    }
  }

/* Handle the clock tick */
    if (Ticked)
    {
      Ticked = 0;
    }

  } // End of main loop
} // End of main


void Timer0(void) interrupt 1 using 1
/* Handle Timer0 interrupts */
{
  Ticked += 1;
}


void Serial(void) interrupt 4 using 2
{
}


void Delay(dword Wait)
/* Delay for 'Wait' TUnits, using idle loop */
{
  word Inner;

  while(Wait--)
  {
    Inner = DELAY_FACTOR;
    while(Inner--);
  }
}  /* Delay() */


void SDelay(void)
/* Pauses for SDELAY loops */
{
  byte Inner;

  Inner = SDELAY;
  while (Inner--);
}